"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionTypes = exports.actions = exports.createActions = exports.getFormHandlers = exports.formHandler = exports.getFormEvents = exports.formEvent = exports.onSubmit = exports.SUBMIT = exports.onFocus = exports.FOCUS = exports.onDragLeave = exports.DRAG_LEAVE = exports.onDragEnter = exports.DRAG_ENTER = exports.onInput = exports.onChange = exports.CHANGE = exports.onBlur = exports.BLUR = exports.getFieldEvents = exports.applyPrefix = exports.fieldEvent = exports.valid = exports.VALID = exports.savedProgress = exports.saveProgress = exports.SAVED_PROGRESS = exports.saved = exports.SAVED = exports.save = exports.SAVE = exports.open = exports.OPEN = exports.meta = exports.META = exports.invalid = exports.INVALID = exports.error = exports.ERROR = exports.close = exports.CLOSE = exports.clearError = exports.CLEAR_ERROR = exports.clear = exports.CLEAR = void 0;

var _set2 = _interopRequireDefault(require("lodash/fp/set"));

var _replace2 = _interopRequireDefault(require("lodash/fp/replace"));

var _memoize2 = _interopRequireDefault(require("lodash/fp/memoize"));

var _mapValues2 = _interopRequireDefault(require("lodash/fp/mapValues"));

var _mapKeys2 = _interopRequireDefault(require("lodash/fp/mapKeys"));

var _join2 = _interopRequireDefault(require("lodash/fp/join"));

var _flow2 = _interopRequireDefault(require("lodash/fp/flow"));

var _curry2 = _interopRequireDefault(require("lodash/fp/curry"));

var _constant2 = _interopRequireDefault(require("lodash/fp/constant"));

var _understory = require("understory");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var basicAction = (0, _curry2.default)(function (type, prefix) {
  return {
    type: type,
    meta: {
      prefix: (0, _utils.createPrefix)(prefix)
    }
  };
});
var noopAction = (0, _curry2.default)(function (type, prefix) {
  return (0, _constant2.default)(basicAction(type, prefix));
});
var createAction = (0, _curry2.default)(function (type, prefix, payload) {
  return (0, _set2.default)('payload', (0, _utils.createPayload)(payload), basicAction(type, prefix));
}); // Close the field. Reset all values to default.

var CLEAR = 'field/CLEAR';
exports.CLEAR = CLEAR;
var clear = basicAction(CLEAR); // Reset error to null.

exports.clear = clear;
var CLEAR_ERROR = 'field/CLEAR_ERROR';
exports.CLEAR_ERROR = CLEAR_ERROR;
var clearError = basicAction(CLEAR_ERROR); // The field has been closed.

exports.clearError = clearError;
var CLOSE = 'field/CLOSE';
exports.CLOSE = CLOSE;
var close = basicAction(CLOSE); // Async error result. Sync errors should be calculated in container. See derivedState().

exports.close = close;
var ERROR = 'field/ERROR';
exports.ERROR = ERROR;
var error = createAction(ERROR); // Set values that are known invalid.

exports.error = error;
var INVALID = 'field/INVALID';
exports.INVALID = INVALID;
var invalid = createAction(INVALID); // Set metadata about the editing process. If you need a place to put extra information.

exports.invalid = invalid;
var META = 'field/META';
exports.META = META;
var meta = createAction(META); // Almost always the first thing that is called.

exports.meta = meta;
var OPEN = 'field/OPEN';
exports.OPEN = OPEN;
var open = createAction(OPEN); // Saving to server.

exports.open = open;
var SAVE = 'field/SAVE';
exports.SAVE = SAVE;
var save = createAction(SAVE); // Has been saved on server.

exports.save = save;
var SAVED = 'field/SAVED';
exports.SAVED = SAVED;
var saved = createAction(SAVED); // Record upload/save progress.

exports.saved = saved;
var SAVED_PROGRESS = 'field/SAVED_PROGRESS';
exports.SAVED_PROGRESS = SAVED_PROGRESS;
var saveProgress = (0, _curry2.default)(function (prefix, valueOrEvent) {
  return createAction(SAVED_PROGRESS, prefix, (0, _utils.getProgress)(valueOrEvent));
});
exports.saveProgress = saveProgress;
var savedProgress = (0, _curry2.default)(function (prefix, valueOrEvent) {
  var action = saveProgress(prefix, valueOrEvent);
  var progress = action.payload;
  return function (dispatch) {
    return progress % 5 === 0 && dispatch(action(prefix, progress)) || false;
  };
}); // Similar to invalid. Save that a value is valid.

exports.savedProgress = savedProgress;
var VALID = 'field/VALID';
exports.VALID = VALID;
var valid = createAction(VALID); // Redux-field events:

exports.valid = valid;
var fieldEvent = {
  clear: clear,
  clearError: clearError,
  close: close,
  error: error,
  invalid: invalid,
  meta: meta,
  open: open,
  save: save,
  saved: saved,
  savedProgress: savedProgress,
  valid: valid
};
exports.fieldEvent = fieldEvent;

var applyPrefix = function applyPrefix(actions) {
  return (0, _flow2.default)(_utils.createPrefix, (0, _join2.default)('.'), (0, _memoize2.default)(function (prefix) {
    return (0, _mapValues2.default)((0, _understory.callWith)(prefix), actions);
  }));
};

exports.applyPrefix = applyPrefix;
var getFieldEvents = applyPrefix(fieldEvent); // FORM & FOCUS EVENTS

exports.getFieldEvents = getFieldEvents;
var BLUR = 'field/BLUR';
exports.BLUR = BLUR;
var onBlur = createAction(BLUR); // On every change of field value.

exports.onBlur = onBlur;
var CHANGE = 'field/CHANGE';
exports.CHANGE = CHANGE;
var onChange = createAction(CHANGE);
exports.onChange = onChange;
var onInput = onChange;
exports.onInput = onInput;
var DRAG_ENTER = 'field/DRAG_ENTER';
exports.DRAG_ENTER = DRAG_ENTER;
var onDragEnter = createAction(DRAG_ENTER);
exports.onDragEnter = onDragEnter;
var DRAG_LEAVE = 'field/DRAG_LEAVE';
exports.DRAG_LEAVE = DRAG_LEAVE;
var onDragLeave = createAction(DRAG_LEAVE); // When a user clicks on a field to edit it.

exports.onDragLeave = onDragLeave;
var FOCUS = 'field/FOCUS';
exports.FOCUS = FOCUS;
var onFocus = noopAction(FOCUS); // Submit, close, save.

exports.onFocus = onFocus;
var SUBMIT = 'field/SUBMIT'; // @TODO Create timer to trigger error if doesn't submit.

exports.SUBMIT = SUBMIT;
var onSubmit = createAction(SUBMIT); // Form and focus events:

exports.onSubmit = onSubmit;
var formEvent = {
  onBlur: onBlur,
  onChange: onChange,
  onFocus: onFocus,
  onInput: onInput,
  onSubmit: onSubmit
};
exports.formEvent = formEvent;
var getFormEvents = applyPrefix(formEvent);
exports.getFormEvents = getFormEvents;
var formHandler = (0, _mapKeys2.default)((0, _replace2.default)('on', 'handle'), formEvent);
exports.formHandler = formHandler;
var getFormHandlers = applyPrefix(formHandler); // Save and reuse these.

exports.getFormHandlers = getFormHandlers;

var createActions = function createActions(prefix) {
  return {
    fieldEvent: getFieldEvents(prefix),
    formEvent: getFormEvents(prefix),
    formHandler: getFormHandlers(prefix)
  };
};

exports.createActions = createActions;
var actions = createActions('default');
exports.actions = actions;
var actionTypes = {
  CLEAR: CLEAR,
  CLEAR_ERROR: CLEAR_ERROR,
  CLOSE: CLOSE,
  ERROR: ERROR,
  INVALID: INVALID,
  META: META,
  OPEN: OPEN,
  SAVE: SAVE,
  SAVED: SAVED,
  SAVED_PROGRESS: SAVED_PROGRESS,
  VALID: VALID,
  BLUR: BLUR,
  CHANGE: CHANGE,
  DRAG_ENTER: DRAG_ENTER,
  DRAG_LEAVE: DRAG_LEAVE,
  FOCUS: FOCUS,
  SUBMIT: SUBMIT
};
exports.actionTypes = actionTypes;