"use strict";

var _keys2 = _interopRequireDefault(require("lodash/fp/keys"));

var _actions = require("./actions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/* globals describe test expect */
describe('clear', function () {
  test('one arg return', function () {
    expect((0, _actions.clear)('default')).toEqual({
      meta: {
        prefix: ['default']
      },
      type: _actions.actionTypes.CLEAR
    });
  });
  test('noop', function () {
    expect((0, _actions.clear)('foo', 'bar')).toEqual({
      meta: {
        prefix: ['foo']
      },
      type: _actions.actionTypes.CLEAR
    });
  });
});
describe('clearError', function () {
  test('one arg return', function () {
    expect((0, _actions.clearError)('foo')).toEqual({
      meta: {
        prefix: ['foo']
      },
      type: _actions.actionTypes.CLEAR_ERROR
    });
  });
  test('noop', function () {
    expect((0, _actions.clearError)('fieldId', 'bar')).toEqual({
      meta: {
        prefix: ['fieldId']
      },
      type: _actions.actionTypes.CLEAR_ERROR
    });
  });
});
describe('close', function () {
  test('one arg return', function () {
    expect((0, _actions.close)('foo')).toEqual({
      meta: {
        prefix: ['foo']
      },
      type: _actions.actionTypes.CLOSE
    });
  });
  test('noop', function () {
    expect((0, _actions.close)('fieldId', 'bar')).toEqual({
      meta: {
        prefix: ['fieldId']
      },
      type: _actions.actionTypes.CLOSE
    });
  });
});
describe('onChange', function () {
  test('one arg return func', function () {
    expect(_typeof((0, _actions.onChange)(['login', 'email']))).toBe('function');
    expect((0, _actions.onChange)(['login', 'email'])('a')).toEqual({
      meta: {
        prefix: ['login', 'email']
      },
      type: _actions.actionTypes.CHANGE,
      payload: 'a'
    });
  });
  test('prefix and payload at same time', function () {
    expect((0, _actions.onChange)(['login'], 'ab')).toEqual({
      meta: {
        prefix: ['login']
      },
      type: _actions.actionTypes.CHANGE,
      payload: 'ab'
    });
  });
});
describe('onFocus', function () {
  test('one arg return func', function () {
    expect(_typeof((0, _actions.onFocus)('example'))).toBe('function');
    var action = {
      meta: {
        prefix: ['example']
      },
      type: _actions.actionTypes.FOCUS
    };
    expect((0, _actions.onFocus)('example')('a')).toEqual(action);
  });
});
describe('meta', function () {
  test('meta is payload object', function () {
    expect(_typeof((0, _actions.meta)(null))).toBe('function');
    expect((0, _actions.meta)(null)({
      extra: 'foo'
    })).toEqual({
      meta: {
        prefix: ['default']
      },
      type: _actions.actionTypes.META,
      payload: {
        extra: 'foo'
      }
    });
  });
});
describe('getFormEvents', function () {
  var actionCreators1 = (0, _actions.getFormEvents)('foo.bar');
  var actionCreators2 = (0, _actions.getFormEvents)(['foo', 'bar']);
  var actionCreators3 = (0, _actions.getFormEvents)('foo.baz');
  var actionCreators4 = (0, _actions.getFormEvents)('foo.bar');
  test('handler for for every input trigger', function () {
    expect((0, _keys2.default)(actionCreators1)).toEqual(['onBlur', 'onChange', 'onFocus', 'onInput', 'onSubmit']);
  });
  test('memoize based on prefix', function () {
    expect(actionCreators1).toBe(actionCreators2);
    expect(actionCreators3).not.toBe(actionCreators2);
    expect(actionCreators2).toBe(actionCreators4);
  });
});