"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = reducer;
exports.fieldReducer = exports.reducers = exports.metaReducer = exports.submitReducer = exports.saveProgressReducer = exports.savedReducer = exports.setSaved = exports.openReducer = exports.dragLeaveReducer = exports.dragEnterReducer = exports.errorReducer = exports.clearError = exports.applyError = exports.closeReducer = exports.blurReducer = exports.changeReducer = exports.touched = exports.setValue = exports.isNewValue = exports.saving = exports.setClose = exports.setFocus = exports.getDragCount = exports.resetFields = exports.defaultState = void 0;

var _update2 = _interopRequireDefault(require("lodash/fp/update"));

var _set2 = _interopRequireDefault(require("lodash/fp/set"));

var _pick2 = _interopRequireDefault(require("lodash/fp/pick"));

var _negate2 = _interopRequireDefault(require("lodash/fp/negate"));

var _merge2 = _interopRequireDefault(require("lodash/fp/merge"));

var _isUndefined2 = _interopRequireDefault(require("lodash/fp/isUndefined"));

var _isEmpty2 = _interopRequireDefault(require("lodash/fp/isEmpty"));

var _isArray2 = _interopRequireDefault(require("lodash/fp/isArray"));

var _gte2 = _interopRequireDefault(require("lodash/fp/gte"));

var _getOr2 = _interopRequireDefault(require("lodash/fp/getOr"));

var _get2 = _interopRequireDefault(require("lodash/fp/get"));

var _flow2 = _interopRequireDefault(require("lodash/fp/flow"));

var _defaultTo2 = _interopRequireDefault(require("lodash/fp/defaultTo"));

var _add2 = _interopRequireDefault(require("lodash/fp/add"));

var _understory = require("understory");

var _prairie = require("prairie");

var _capeRedux = require("cape-redux");

var _actions = require("./actions");

var _reducers;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Only keeping state we can not calculate. See derivedState().
var defaultState = {
  blur: false,
  // When true the field is open but does not have focus.
  dragCount: 0,
  // For keeping track of entering children but maintaining focus.
  error: null,
  // String usually. Could be object for more complex error.
  focus: false,
  // When true the field is open and it has focus.
  id: null,
  // String. Used as a unique key/id for this specific field value.
  initialValue: null,
  // Anything.
  invalid: {},
  // index of invalid values.
  meta: null,
  // Anything.
  savedProgress: 0,
  // Percentage Number 0-99.
  savedValue: null,
  isSaving: false,
  // Bool.
  isTouched: false,
  valid: {},
  // index of valid values.
  value: null // Anything.

};
exports.defaultState = defaultState;

var resetFields = function resetFields(fields) {
  return (0, _prairie.mergeWith)((0, _pick2.default)(fields, defaultState));
};

exports.resetFields = resetFields;
var getDragCount = (0, _get2.default)('dragCount');
exports.getDragCount = getDragCount;
var setFocus = (0, _prairie.mergeWith)({
  blur: false,
  focus: true,
  isTouched: true
});
exports.setFocus = setFocus;
var setClose = resetFields(['blur', 'focus']);
exports.setClose = setClose;
var saving = (0, _set2.default)('isSaving', true);
exports.saving = saving;

var isNewValue = function isNewValue(state, payload) {
  return !(0, _isUndefined2.default)(payload) && payload !== state.value;
};

exports.isNewValue = isNewValue;
var setValue = (0, _understory.overBranch)(isNewValue, (0, _prairie.setIn)('value'));
exports.setValue = setValue;
var touched = (0, _understory.overBranch)((0, _negate2.default)((0, _get2.default)('isTouched')), (0, _set2.default)('isTouched', true));
exports.touched = touched;
var changeReducer = (0, _flow2.default)(setValue, touched);
exports.changeReducer = changeReducer;
var blurReducer = (0, _flow2.default)(changeReducer, resetFields(['dragCount', 'focus']), (0, _set2.default)('blur', true));
exports.blurReducer = blurReducer;
var closeReducer = (0, _flow2.default)(setClose, touched);
exports.closeReducer = closeReducer;
var applyError = (0, _flow2.default)((0, _prairie.setIn)('error'), touched);
exports.applyError = applyError;
var clearError = (0, _set2.default)('error', defaultState.error);
exports.clearError = clearError;

var errorReducer = function errorReducer(state, payload) {
  if (payload.error && payload.value) {
    return setValue(applyError(state, payload.error), payload.value);
  }

  return applyError(state, payload);
};

exports.errorReducer = errorReducer;
var dragEnterReducer = (0, _flow2.default)(setFocus, (0, _update2.default)('dragCount', (0, _add2.default)(1)));
exports.dragEnterReducer = dragEnterReducer;
var dragLeaveReducer = (0, _flow2.default)((0, _update2.default)('dragCount', (0, _understory.subtrahend)(1)), (0, _understory.overBranch)((0, _flow2.default)(getDragCount, (0, _gte2.default)(0)), blurReducer)); // Can not overwrite value.

exports.dragLeaveReducer = dragLeaveReducer;

var openReducer = function openReducer(state) {
  var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return _objectSpread({}, state, {
    focus: true,
    id: (0, _defaultTo2.default)((0, _defaultTo2.default)(null, state.id), payload.id),
    initialValue: (0, _defaultTo2.default)((0, _defaultTo2.default)((0, _defaultTo2.default)(null, state.value), state.initialValue), payload.initialValue),
    isTouched: true,
    value: (0, _defaultTo2.default)((0, _defaultTo2.default)(null, payload.initialValue), state.value)
  });
};

exports.openReducer = openReducer;
var setSaved = resetFields(['error', 'isSaving', 'savedProgress']);
exports.setSaved = setSaved;

var savedReducer = function savedReducer(state, payload) {
  return _objectSpread({}, setSaved(state), {
    id: (0, _getOr2.default)(state.id, 'id', payload),
    initialValue: state.value,
    savedValue: (0, _getOr2.default)(state.value, 'value', payload),
    value: (0, _getOr2.default)(state.value, 'value', payload)
  });
};

exports.savedReducer = savedReducer;
var saveProgressReducer = (0, _flow2.default)((0, _prairie.setIn)('savedProgress'), saving);
exports.saveProgressReducer = saveProgressReducer;
var submitReducer = (0, _flow2.default)(setValue, setClose, saving);
exports.submitReducer = submitReducer;

var metaReducer = function metaReducer(_ref, payload) {
  var meta = _ref.meta,
      state = _objectWithoutProperties(_ref, ["meta"]);

  return _objectSpread({}, state, {
    isTouched: true,
    meta: (0, _merge2.default)(meta, payload)
  });
};

exports.metaReducer = metaReducer;
var reducers = (_reducers = {}, _defineProperty(_reducers, _actions.CLEAR, function () {
  return defaultState;
}), _defineProperty(_reducers, _actions.CLEAR_ERROR, clearError), _defineProperty(_reducers, _actions.CLOSE, closeReducer), _defineProperty(_reducers, _actions.ERROR, errorReducer), _defineProperty(_reducers, _actions.INVALID, function (state, payload) {
  return (0, _set2.default)(['invalid', payload.key], payload.value, state);
}), _defineProperty(_reducers, _actions.META, metaReducer), _defineProperty(_reducers, _actions.OPEN, openReducer), _defineProperty(_reducers, _actions.SAVE, saving), _defineProperty(_reducers, _actions.SAVED_PROGRESS, saveProgressReducer), _defineProperty(_reducers, _actions.SAVED, savedReducer), _defineProperty(_reducers, _actions.VALID, function (state, payload) {
  return (0, _set2.default)(['valid', payload.key], payload.value, state);
}), _defineProperty(_reducers, _actions.BLUR, blurReducer), _defineProperty(_reducers, _actions.CHANGE, changeReducer), _defineProperty(_reducers, _actions.DRAG_ENTER, dragEnterReducer), _defineProperty(_reducers, _actions.DRAG_LEAVE, dragLeaveReducer), _defineProperty(_reducers, _actions.FOCUS, setFocus), _defineProperty(_reducers, _actions.SUBMIT, submitReducer), _reducers);
exports.reducers = reducers;
var fieldReducer = (0, _capeRedux.createReducer)(reducers, defaultState, {
  skipErrors: false
});
exports.fieldReducer = fieldReducer;

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments.length > 1 ? arguments[1] : undefined;
  if ((0, _capeRedux.noReducerOfType)(reducers)(action)) return state;

  if (!(0, _isArray2.default)(action.meta.prefix) || (0, _isEmpty2.default)(action.meta.prefix)) {
    throw new Error('Action must contain meta.prefix array.');
  } // Get the state slice we need for this action.


  var oldFieldState = (0, _get2.default)(action.meta.prefix, state);
  var newFieldState = fieldReducer(oldFieldState, action);
  if (oldFieldState === newFieldState) return state; // console.log(get('value', oldFieldState), newFieldState.value)

  return (0, _set2.default)(action.meta.prefix, newFieldState, state);
}