"use strict";

var _ = _interopRequireWildcard(require("."));

var _reducer = require("./reducer");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* globals describe test expect */
describe('setValue', function () {
  test('leave state untouched when undefined new value', function () {
    var state = {};
    expect((0, _reducer.setValue)(state)).toBe(state);
    expect((0, _reducer.setValue)(state, undefined)).toBe(state);
    expect((0, _reducer.setValue)(state, null)).not.toBe(state);
  });
  test('leave state untouched when value matches state', function () {
    var state = {
      value: 'foo'
    };
    expect((0, _reducer.setValue)(state, 'foo')).toBe(state);
  });
  test('replace state when new value', function () {
    var state = {
      value: 'foo'
    };
    var update = (0, _reducer.setValue)(state, 'bar');
    expect(update).not.toBe(state);
    expect(update).toEqual({
      value: 'bar'
    });
  });
});
describe('getDragCount', function () {
  test('sets blur and focus to false', function () {
    expect((0, _reducer.getDragCount)(_reducer.defaultState)).toBe(0);
  });
});
describe('blurReducer', function () {
  test('update value', function () {
    var state = {
      blur: false,
      dragCount: 2,
      value: 'foo'
    };
    expect((0, _reducer.blurReducer)(state, 'bar')).toEqual({
      blur: true,
      dragCount: 0,
      focus: false,
      isTouched: true,
      value: 'bar'
    });
    expect((0, _reducer.blurReducer)({
      blur: false,
      dragCount: 2,
      value: null
    }, '')).toEqual({
      blur: true,
      dragCount: 0,
      focus: false,
      isTouched: true,
      value: ''
    });
  });
  test('blur reducer', function () {
    var state = (0, _.default)(undefined, (0, _.onBlur)('example', {
      nativeEvent: {},
      target: {}
    }));
    expect(state).toEqual({
      example: _objectSpread({}, _reducer.defaultState, {
        value: '',
        blur: true,
        isTouched: true
      })
    });
  });
});
describe('openReducer', function () {
  var res = {
    focus: true,
    id: null,
    initialValue: null,
    isTouched: true,
    value: null
  };
  test('set focus and touched', function () {
    expect((0, _reducer.openReducer)({}, {})).toEqual(res);
  });
  test('leave id alone', function () {
    expect((0, _reducer.openReducer)({
      id: 1
    }, {})).toEqual(_objectSpread({}, res, {
      id: 1
    }));
  });
  test('set id on open', function () {
    expect((0, _reducer.openReducer)({
      id: 1
    }, {
      id: 2
    })).toEqual(_objectSpread({}, res, {
      id: 2
    }));
  });
  test('overwrite initialValue', function () {
    expect((0, _reducer.openReducer)({
      value: 1,
      initialValue: 1
    }, {
      initialValue: 2
    })).toEqual(_objectSpread({}, res, {
      value: 1,
      initialValue: 2
    }));
  });
  test('no overwrite of value on open', function () {
    expect((0, _reducer.openReducer)({
      value: 1
    }, {
      value: 2
    })).toEqual(_objectSpread({}, res, {
      value: 1,
      initialValue: 1
    }));
  });
  test('set value from initVal', function () {
    expect((0, _reducer.openReducer)({}, {
      initialValue: 2
    })).toEqual(_objectSpread({}, res, {
      value: 2,
      initialValue: 2
    }));
  });
});
describe('closeReducer', function () {
  var res = {
    blur: false,
    focus: false,
    isTouched: true
  };
  test('set touched', function () {
    expect((0, _reducer.closeReducer)({}, {})).toEqual(res);
  });
  test('leave value alone. Do not use any values from action.', function () {
    expect((0, _reducer.closeReducer)({
      value: 1
    }, {
      value: 2,
      id: 4
    })).toEqual(_objectSpread({}, res, {
      value: 1
    }));
  });
});
describe('changeReducer', function () {
  test('replace when untouched but value match', function () {
    var state = {
      value: 'foo'
    };
    var change = (0, _reducer.changeReducer)(state, 'foo');
    expect(change).not.toBe(state);
    expect(change).toEqual({
      value: 'foo',
      isTouched: true
    });
  });
  var state = {
    value: 'foo',
    isTouched: true
  };
  test('ignore when touched and value match', function () {
    var change = (0, _reducer.changeReducer)(state, 'foo');
    expect(change).toBe(state);
  });
  test('replace when new value', function () {
    var change = (0, _reducer.changeReducer)(state, 'bar');
    expect(change).not.toBe(state);
    expect(change).toEqual({
      value: 'bar',
      isTouched: true
    });
  });
});
describe('reducer open close', function () {
  var openState = _objectSpread({}, _reducer.defaultState, {
    focus: true,
    isTouched: true
  });

  var state = (0, _.default)(undefined, (0, _.open)(null, undefined));
  test('set focus touched', function () {
    expect(state.default).toEqual(openState);
  });
  test('set value from initVal', function () {
    expect((0, _.default)(undefined, (0, _.open)(null, {
      initialValue: 2
    })).default).toEqual(_objectSpread({}, openState, {
      value: 2,
      initialValue: 2
    }));
    expect((0, _.default)(state, (0, _.open)(null, {
      initialValue: 2
    })).default).toEqual(_objectSpread({}, openState, {
      value: 2,
      initialValue: 2
    }));
  });
  test('open and close state', function () {
    state = (0, _.default)(state, (0, _.close)('default'));
    expect(state.default).toEqual(_objectSpread({}, _reducer.defaultState, {
      isTouched: true
    }));
    state = (0, _.default)(state, (0, _.open)(null, {
      initialValue: 'bar',
      value: 'foo'
    }));
    expect(state.default).toEqual(_objectSpread({}, openState, {
      initialValue: 'bar',
      value: 'bar'
    }));
  });
});
describe('focusReducer', function () {
  var res = {
    blur: false,
    focus: true,
    isTouched: true
  };
  test('sets blur and focus to false', function () {
    expect((0, _reducer.setFocus)({
      blur: true,
      focus: true
    })).toEqual(res);
    expect((0, _reducer.setFocus)({})).toEqual(res);
  });
  test('blur reducer will toggle blur/focus', function () {
    var res2 = (0, _reducer.blurReducer)(res);
    expect(res2).toEqual({
      blur: true,
      dragCount: 0,
      focus: false,
      isTouched: true
    });
    expect((0, _reducer.setFocus)(res2)).toEqual(_objectSpread({}, res, {
      dragCount: 0
    }));
  });
  test('leave value alone', function () {
    var res2 = {
      blur: false,
      focus: true,
      isTouched: true,
      value: ''
    };
    expect((0, _reducer.setFocus)({
      value: ''
    })).toEqual(res2);
    expect((0, _reducer.setFocus)((0, _reducer.blurReducer)(res2, ''))).toEqual(_objectSpread({}, res2, {
      dragCount: 0
    }));
  });
  test('focus action', function () {
    var state = (0, _.default)(undefined, (0, _.onFocus)('example')());
    expect(state).toEqual({
      example: _objectSpread({}, _reducer.defaultState, res)
    });
    state = (0, _.default)(state, (0, _.onBlur)('example', {
      nativeEvent: {},
      target: {}
    }));
    expect(state).toEqual({
      example: _objectSpread({}, _reducer.defaultState, {
        value: '',
        blur: true,
        isTouched: true
      })
    });
    expect(state.example.value).toBe('');
    state = (0, _.default)(state, (0, _.onFocus)('example')());
    expect(state.example.value).toBe('');
    expect(state).toEqual({
      example: _objectSpread({}, _reducer.defaultState, res, {
        value: ''
      })
    });
  });
});
describe('applyError', function () {
  var errStr = 'error msg';
  var res = {
    error: errStr,
    isTouched: true
  };
  test('add error string', function () {
    expect((0, _reducer.applyError)({}, errStr)).toEqual(res);
  });
  test('overwrite error', function () {
    expect((0, _reducer.applyError)({}, 'junk')).toEqual({
      error: 'junk',
      isTouched: true
    });
  });
});
describe('errorReducer', function () {
  var errStr = 'error msg';
  var res = {
    error: errStr,
    isTouched: true
  };
  test('add error string when payload is string', function () {
    expect((0, _reducer.errorReducer)({}, errStr)).toEqual(res);
  });
  test('add error string and value when payload is obj', function () {
    expect((0, _reducer.errorReducer)({}, {
      error: errStr,
      value: 'bas'
    })).toEqual({
      error: errStr,
      value: 'bas',
      isTouched: true
    });
  });
});