"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getErrorVal = getErrorVal;
exports.getStatus = getStatus;
exports.derivedState = derivedState;
exports.calcFieldState = exports.selectFieldValue = exports.selectField = exports.getFormState = exports.fieldSelector = exports.prependKey = exports.selectPrefix = exports.getPrefix = exports.getFieldSelector = exports.selectForm = exports.REDUCER_KEY = void 0;

var _partial2 = _interopRequireDefault(require("lodash/fp/partial"));

var _nthArg2 = _interopRequireDefault(require("lodash/fp/nthArg"));

var _isFunction2 = _interopRequireDefault(require("lodash/fp/isFunction"));

var _identity2 = _interopRequireDefault(require("lodash/fp/identity"));

var _getOr2 = _interopRequireDefault(require("lodash/fp/getOr"));

var _get2 = _interopRequireDefault(require("lodash/fp/get"));

var _flow2 = _interopRequireDefault(require("lodash/fp/flow"));

var _find2 = _interopRequireDefault(require("lodash/fp/find"));

var _concat2 = _interopRequireDefault(require("lodash/fp/concat"));

var _at2 = _interopRequireDefault(require("lodash/fp/at"));

var _capeSelect = require("cape-select");

var _reducer = require("./reducer");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var REDUCER_KEY = 'form';
exports.REDUCER_KEY = REDUCER_KEY;
var selectForm = (0, _get2.default)(REDUCER_KEY); // fieldSelector(prefixArray)(state)

exports.selectForm = selectForm;
var getFieldSelector = (0, _partial2.default)(_capeSelect.select, [selectForm]);
exports.getFieldSelector = getFieldSelector;

function getErrorVal(state, _ref) {
  var pristine = _ref.pristine,
      validate = _ref.validate;
  if (pristine && !state.error) return null;

  if ((0, _isFunction2.default)(validate)) {
    var errorVal = validate(state.value);
    if (errorVal) return errorVal;
  }

  if (state.invalid[state.value]) return state.invalid[state.value];
  return state.error;
}

function getStatus(errorVal, isValid) {
  if (isValid) return 'success';
  if (!errorVal) return null;
  return errorVal.status || 'error';
} // Validate function should return a string or object
// error: { message: String, suggestion: String, status: String }


function derivedState(state, initialValue, validate) {
  var initVal = state.initialValue || initialValue || _reducer.defaultState.initialValue;
  var pristine = state.value === initVal;
  var errorVal = getErrorVal(state, {
    pristine: pristine,
    validate: validate
  });
  var isValid = !errorVal && !pristine;
  var isOpen = state.blur || state.focus;
  return _objectSpread({}, state, {
    errorMessage: errorVal && errorVal.message ? errorVal.message : errorVal,
    hasError: !!errorVal,
    initialValue: initVal,
    invalidValue: state.invalid[state.value] || null,
    isClosed: !isOpen,
    isDirty: !pristine,
    isEditing: state.focus && !pristine,
    isOpen: isOpen,
    isPristine: pristine,
    isSaved: !pristine && state.value === state.savedValue,
    isValid: isValid,
    status: getStatus(errorVal, isValid),
    suggestion: errorVal && errorVal.suggestion ? errorVal.suggestion : null,
    validValue: state.valid[state.value] || null
  });
} // export function calcState(state) {
// }
// The new core of selecting state based on prefix.


var getPrefix = (0, _flow2.default)((0, _at2.default)(['prefix', 'id']), (0, _find2.default)(_identity2.default), _utils.createPrefix);
exports.getPrefix = getPrefix;
var selectPrefix = (0, _flow2.default)((0, _nthArg2.default)(1), getPrefix);
exports.selectPrefix = selectPrefix;
var prependKey = (0, _concat2.default)(REDUCER_KEY);
exports.prependKey = prependKey;
var fieldSelector = (0, _flow2.default)(_utils.createPrefix, prependKey, (0, _getOr2.default)(_reducer.defaultState));
exports.fieldSelector = fieldSelector;
var getFormState = (0, _flow2.default)(getPrefix, fieldSelector);
exports.getFormState = getFormState;

var selectField = function selectField(state, props) {
  return getFormState(props)(state);
};

exports.selectField = selectField;
var selectFieldValue = (0, _flow2.default)(selectField, (0, _get2.default)('value'));
exports.selectFieldValue = selectFieldValue;
var calcFieldState = (0, _flow2.default)(selectField, derivedState); // // Older stuff
// // Please note that it will return defaultState if there is an invalid prefix.
// export function selectFieldState(state, prefix, selectFormState = selectForm) {
//   return get(selectFormState(state), createPrefix(prefix), defaultState)
// }
// // Select prefix and selectForm from props.
// export function getFieldState(state, props) {
//   return selectFieldState(state, props.prefix, props.selectForm)
// }
// export const getState = createSelector(
//   getFieldState, getProp('initialValue'), getProp('validate'), derivedState
// )
// export function getFieldValue(state, prefix, selectFormState, prop = 'value') {
//   return get(selectFieldState(state, prefix, selectFormState), prop)
// }
// // Returns selector.
// export function fieldValue(prefix, prop = 'value') {
//   return state => get(selectFieldState(state, prefix), prop)
// }

exports.calcFieldState = calcFieldState;