"use strict";

var _select = require("./select");

var _reducer = require("./reducer");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/* globals describe test expect */
describe('prependKey', function () {
  test('prepend form onto array', function () {
    expect((0, _select.prependKey)([])).toEqual(['form']);
    expect((0, _select.prependKey)(['foo'])).toEqual(['form', 'foo']);
  });
});
describe('fieldSelector', function () {
  test('returns array with form added to front', function () {
    var selector = (0, _select.fieldSelector)('foo');
    expect(_typeof(selector)).toBe('function');
    var fieldState = selector({
      form: {}
    });
    expect(fieldState).toBe(_reducer.defaultState);
    var state = {
      form: {
        foo: {
          value: ''
        }
      }
    };
    var fieldState2 = selector(state);
    expect(fieldState2).toBe(state.form.foo);
  });
});
describe('calcFieldState', function () {
  var state = {
    form: {
      foo: {
        value: ''
      }
    }
  };
  test('returns field of prefix', function () {
    var res = (0, _select.calcFieldState)(state, {
      id: 'bar'
    });
    expect(res).toEqual({
      blur: false,
      dragCount: 0,
      error: null,
      errorMessage: null,
      focus: false,
      hasError: false,
      id: null,
      initialValue: null,
      invalid: {},
      invalidValue: null,
      isClosed: true,
      isDirty: false,
      isEditing: false,
      isOpen: false,
      isPristine: true,
      isSaved: false,
      isSaving: false,
      isTouched: false,
      isValid: false,
      meta: null,
      savedProgress: 0,
      savedValue: null,
      status: null,
      suggestion: null,
      valid: {},
      validValue: null,
      value: null
    });
  });
});
describe('getPrefix', function () {
  test('returns prefix prop value', function () {
    expect((0, _select.getPrefix)({
      prefix: 'foo'
    })).toEqual(['foo']);
    expect((0, _select.getPrefix)({
      prefix: 'bar.foo',
      other: 'baz'
    })).toEqual(['bar', 'foo']);
  });
  test('returns prefix prop before id value', function () {
    expect((0, _select.getPrefix)({
      prefix: 'foo',
      id: 'baz'
    })).toEqual(['foo']);
    expect((0, _select.getPrefix)({
      prefix: 'bar',
      id: 'ling',
      other: 'baz'
    })).toEqual(['bar']);
  });
  test('returns id prop value when prefix is falsey', function () {
    expect((0, _select.getPrefix)({
      prefix: null,
      id: 'baz'
    })).toEqual(['baz']);
    expect((0, _select.getPrefix)({
      id: 'ling',
      other: 'baz'
    })).toEqual(['ling']);
    expect((0, _select.getPrefix)({
      id: 'ding.ling',
      prefix: false
    })).toEqual(['ding', 'ling']);
  });
});