"use strict";

var _utils = require("./utils");

// import { flow } from 'lodash/fp'

/* globals describe test expect */
// test('getProgress', (t) => {
//   t.equal(getProgress({ bytesTransferred: 0, totalBytes: 510 }), 0, 'zero progress')
//   t.end()
// })
describe('progPercent', function () {
  test('divide and round', function () {
    expect((0, _utils.progPercent)([50, 100])).toBe(50);
    expect((0, _utils.progPercent)([100, 510])).toBe(20);
  });
});
describe('eventProg', function () {
  test('event handling', function () {
    expect((0, _utils.eventProg)({
      loaded: 0,
      total: 10
    })).toBe(0);
    expect((0, _utils.eventProg)({
      loaded: 50.4,
      total: 100
    })).toBe(50);
    expect((0, _utils.eventProg)({
      loaded: 256,
      total: 510
    })).toBe(50);
    expect((0, _utils.eventProg)({
      loaded: 100,
      total: 510
    })).toBe(20);
  });
});
describe('fireProg', function () {
  test('firebase handling', function () {
    expect((0, _utils.fireProg)({
      bytesTransferred: 256,
      totalBytes: 510
    })).toBe(50);
    expect((0, _utils.fireProg)({
      bytesTransferred: 100,
      totalBytes: 510
    })).toBe(20);
  });
});
describe('getProgress', function () {
  test('converts string to number and round down', function () {
    expect((0, _utils.getProgress)('10.2')).toBe(10);
    expect((0, _utils.getProgress)('2.7')).toBe(2);
  });
  test('number round down', function () {
    expect((0, _utils.getProgress)(10.2)).toBe(10);
    expect((0, _utils.getProgress)(2.7)).toBe(2);
  });
  test('event handling', function () {
    expect((0, _utils.getProgress)({
      loaded: 50.4,
      total: 100
    })).toBe(50);
    expect((0, _utils.getProgress)({
      loaded: 256,
      total: 510
    })).toBe(50);
    expect((0, _utils.getProgress)({
      loaded: 100,
      total: 510
    })).toBe(20);
  });
  test('firebase handling', function () {
    expect((0, _utils.getProgress)({
      bytesTransferred: 256,
      totalBytes: 510
    })).toBe(50);
    expect((0, _utils.getProgress)({
      bytesTransferred: 100,
      totalBytes: 510
    })).toBe(20);
  });
  test('zero progress', function () {
    expect((0, _utils.getProgress)('0.7')).toBe(0);
    expect((0, _utils.getProgress)(0.7)).toBe(0);
    expect((0, _utils.getProgress)({
      loaded: 0,
      total: 510
    })).toBe(0);
    expect((0, _utils.getProgress)({
      bytesTransferred: 0,
      totalBytes: 510
    })).toBe(0);
  });
});